%orbital elements with J2
clear all
clc
close all

%constants of the problem START%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
G = 6.6742*10^(-20); %km^3/(kg*s^2) gravitational constant
m1 = 5.9726*10^24; %kg mass of Earth
mu = G*m1; %km^3/s^2 gravitational parameter
JulianDate = juliandate([2017,03,03,12,00,00]);%days Julian Date (input is GMT time) -- equivalent to 2017/062/12:00:00.000
gst = gstime(JulianDate); %rad Greenwhich location on julian date (longitude in ECI)
omega_E = 72.9217*10^(-6); %rad/s Earth's angular velocity
%constants of the problem END%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%ISS data from
%http://spaceflight.nasa.gov/realdata/sightings/SSapplications/Post/JavaSSOP/orbit/ISS/SVPOST.html
%@ 2017/062/12:00:00.000

a =  6778928.17*10^(-3); %KM
enorm = .0008446;
inclination =51.46843*pi/180; %rad
RAAN = 203.98784*pi/180; %rad
arg_per = 141.71208*pi/180; %rad
true_anomaly = 149.88842*pi/180; %rad
hnorm = sqrt(a*mu*(1-enorm^2));
T = (2*pi/sqrt(mu))*a^(3/2); %s period

%%%%%FROM ORBITAL ELEMENTS TO STATE VECTOR (START)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
R1_i = [1 0 0; 0 cos(inclination) sin(inclination); 0 -sin(inclination) cos(inclination)];
R3_Om = [cos(RAAN) sin(RAAN) 0; -sin(RAAN) cos(RAAN) 0; 0 0 1];
R3_om = [cos(arg_per) sin(arg_per) 0; -sin(arg_per) cos(arg_per) 0; 0 0 1];

support_var = R3_om*R1_i*R3_Om;
x = support_var(1,1:3);
y = support_var(2,1:3);
z = support_var(3,1:3);

e_orb = enorm*x;
h_orb = hnorm*z;

rnorm = ((hnorm^2)/mu)*(1/(1+enorm*cos(true_anomaly)));
r = (rnorm*cos(true_anomaly)*x + rnorm*sin(true_anomaly)*y)';
u_r = r/norm(r);
u_per = (cross(z, u_r)/norm(cross(z, u_r)))';
v = (mu/hnorm)*enorm*sin(true_anomaly)*u_r + (mu/hnorm)*(1+enorm*cos(true_anomaly))*u_per;
%%%%%FROM ORBITAL ELEMENTS TO STATE VECTOR (END)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%numerical integration of Cartesian State Vector
Y0 = [r; v];
tf = 1*6*3600; %s 
TSPAN = [0 tf]; %duration of integration in seconds
OPTIONS = odeset('Maxstep', 10);
[TOUT,YOUT] = ode45(@relacc_with_j2,TSPAN,Y0,OPTIONS);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure(1)
plot3(YOUT(:,1),YOUT(:,2),YOUT(:,3),'r')
grid on
hold on
plot3(linspace(0,2*a),zeros(100),zeros(100),'k','linewidth',2)
plot3(zeros(100),linspace(0,2*a),zeros(100),'k','linewidth',2)
plot3(zeros(100),zeros(100),linspace(0,2*a),'k','linewidth',2)
axis equal
load('topo.mat', 'topo', 'topomap1');
[X,Y,Z] = sphere;
props.FaceColor= 'texture';
props.EdgeColor = 'none';
props.FaceLighting = 'phong';
props.CData = topo;
s=surface(6378*X,6378*Y,6378*Z,props);
xlabel('x [km]');ylabel('y [km]');zlabel('z [km]')

%orbital elements, latitude and longitude (to be corrected with Earth's
%rotation)
[size1 size2] = size(YOUT);
for i = 1:size1
    [a_out(i), enorm(i), inclination(i), RAAN(i), arg_per(i), true_anomaly(i)] = SV2OE(YOUT(i,1:3), YOUT(i,4:6));
    lat(i) = asin(YOUT(i,3)/norm(YOUT(i,1:3)));
    lon(i) = atan2(YOUT(i,2),YOUT(i,1));
end

figure(2)
plot(TOUT/3600, a_out)
grid on
xlabel('time [hr]')
ylabel('semi-major axis a [km]')

figure(3)
plot(TOUT/3600, enorm)
grid on
xlabel('time [hr]')
ylabel('eccentricity e')

figure(4)
plot(TOUT/3600, inclination*180/pi)
grid on
xlabel('time [hr]')
ylabel('inclination i [deg]')

figure(5)
plot(TOUT/3600, RAAN*180/pi)
grid on
xlabel('time [hr]')
ylabel('RANN \Omega [deg]')

figure(6)
plot(TOUT/3600, arg_per*180/pi)
grid on
xlabel('time [hr]')
ylabel('argument of perigee \omega [deg]')


%correcting longitude
correct_lon = mod((lon' - (gst+omega_E*TOUT)),2*pi); %correcting longitude taking into account Earth's rotation

figure(7)
plot(correct_lon*180/pi, lat*180/pi, 'LineStyle', 'none', 'Marker', '.')
hold on
plot(correct_lon(1)*180/pi, lat(1)*180/pi, 'r*' )
plot(correct_lon(end)*180/pi, lat(end)*180/pi, 'k*' )
xlabel('East longitude [deg]')
ylabel('Latitude [deg]')